@@ -109,9 +109,12 @@ MvQBufrMainPanel::MvQBufrMainPanel(bool filtered,QWidget* parent) :
     connect(messagePanel_,SIGNAL(keyProfileChanged()),
             this,SIGNAL(keyProfileChanged()));
 
-    connect(messagePanel_,SIGNAL(messageSelected(int)),
+    connect(messagePanel_,SIGNAL(messageSelected(int)),             
             this,SLOT(messageSelected(int)));
 
+    connect(messagePanel_,SIGNAL(messageNumDetermined()),
+            this,SIGNAL(messageNumDetermined()));
+
     connect(gotoPanel_,SIGNAL(messageChanged(int)),
             this,SLOT(messageChangedInGoto(int)));
 
@@ -231,7 +234,7 @@ void MvQBufrMainPanel::messageSelected(int msgCnt)
 {
     //Notify the log panel about the selection.
     if(inProfileLoad_ == false)
-        emit newMessageSelected();
+        emit newMessageSelected(msgCnt);
 
     //The message counter might not have been initialised. But at this point
     //we must know the correct message num so we can set the counter.
@@ -244,6 +247,9 @@ void MvQBufrMainPanel::messageSelected(int msgCnt)
     gotoPanel_->setMessageValue(msgCnt+1,false);
 
     MvEccBufrMessage *msg=data_->message(msgCnt);
+    if(!msg)
+        return;
+
     Q_ASSERT(msg);
 
     gotoPanel_->resetSubsetNum(msg->subsetNum(),false);
@@ -1007,16 +1013,19 @@ void MvQBufrDumpPanel::loadTableDump()
 
     QColor hCol(80,80,80);
 
+    std::string masterDir, localDir;
+    msg->tablesDirs(masterDir,localDir);
+
     QString s;
     s+="ecCodes used the following tables to decode the current message: <br><br>";
     s+="<h2><font color=\'" + hCol.name() + "\'>WMO tables</font></h2>";
-    s+="<br>&nbsp;&nbsp;<b>Path</b>: " + QString::fromStdString(msg->tablesMasterDir());
+    s+="<br>&nbsp;&nbsp;<b>Path</b>: " + QString::fromStdString(masterDir);
     s+="<br>&nbsp;&nbsp;<b>Element table</b>: element.table";
     s+="<br>&nbsp;&nbsp;<b>Sequence table</b>: sequence.def";
 
     if(msg->localTablesVersionNumber() != 0)
     {
-        QString localFile=QString::fromStdString(msg->tablesLocalDir());
+        QString localFile=QString::fromStdString(localDir);
         s+="<br><h2><font color=\'" + hCol.name() + "\'>Local tables</font></h2>";
         s+="<br>&nbsp;&nbsp;<b>Path:</b> " + localFile;
         s+="<br>&nbsp;&nbsp;<b>Element table</b>: element.table";
@@ -1335,8 +1344,17 @@ BufrExaminer::BufrExaminer(QWidget *parent) :
     connect(mainPanel_,SIGNAL(keyProfileChanged()),
             this,SLOT(slotKeyProfileChanged()));
 
-    connect(mainPanel_,SIGNAL(newMessageSelected()),
-            logPanel_,SLOT(newMessageLoaded()));
+    connect(mainPanel_,SIGNAL(newMessageSelected(int)),
+            this,SLOT(newMessageLoaded(int)));
+
+    connect(mainPanel_,SIGNAL(newMessageSelected(int)),
+            logPanel_,SLOT(newMessageLoaded(int)));
+
+    connect(mainPanel_,SIGNAL(messageNumDetermined()),
+            this,SLOT(updateFileInfoLabel()));
+
+    connect(mainPanel_,SIGNAL(messageNumDetermined()),
+            this,SLOT(initFilterEditorHelpers()));
 
     //----------------------------
     // Setup menus and toolbars
@@ -1508,8 +1526,11 @@ void BufrExaminer::createFilteredPanel()
     connect(filteredPanel_,SIGNAL(keyProfileChanged()),
             this,SLOT(slotKeyProfileChanged()));
 
-    connect(filteredPanel_,SIGNAL(newMessageSelected()),
-            logPanel_,SLOT(newMessageLoaded()));
+    connect(filteredPanel_,SIGNAL(newMessageSelected(int)),
+            this,SLOT(newMessageLoaded(int)));
+
+    connect(filteredPanel_,SIGNAL(newMessageSelected(int)),
+            logPanel_,SLOT(newMessageLoaded(int)));
 
     if(!filteredData_)
     {
@@ -1542,15 +1563,17 @@ void BufrExaminer::loadFile(QString f)
     //Reload
     mainPanel_->reloadData(false);
 
-    if(sourceFileName_ == f)
-        sourceMessageNum_=data_->totalMessageNum();
+    //if(sourceFileName_ == f)
+    //    sourceMessageNum_=data_->totalMessageNum();
 
     updateFileInfoLabel();
 
     //Notify the the filter dialogue
     if(filterDialog_)
     {
-       filterDialog_->newFileLoaded(f.toStdString());
+        filterDialog_->newFileLoaded(f.toStdString());
+        //helpers will be initailised later since the initial when
+        //there is data already scanned available!!!
     }
 
 #ifdef ECCODES_UI
@@ -1577,8 +1600,8 @@ void BufrExaminer::loadEmptyFile(QString f)
     //Reload
     //mainPanel_->reloadData();
 
-    if(sourceFileName_ == f)
-        sourceMessageNum_=data_->totalMessageNum();
+    //if(sourceFileName_ == f)
+    //    sourceMessageNum_=data_->totalMessageNum();
 
     updateFileInfoLabel();
 #ifdef ECCODES_UI
@@ -1596,6 +1619,8 @@ void BufrExaminer::loadFilteredFile(QString f,MvKeyProfile* extractedProf)
 
     Q_ASSERT(filteredPanel_);
     Q_ASSERT(filteredData_);
+    Q_ASSERT(data_);
+
     filteredPanel_->clear();
 
     //Set the new file name
@@ -1619,7 +1644,7 @@ void BufrExaminer::loadFilteredFile(QString f,MvKeyProfile* extractedProf)
     actionSave_->setEnabled(filteredData_->totalMessageNum() > 0);
 
     //set control panel
-    filteredPanel_->setFilterInfo(true,sourceMessageNum_,filteredData_->totalMessageNum());
+    filteredPanel_->setFilterInfo(true,data_->totalMessageNum(),filteredData_->totalMessageNum());
 
     //updateFileInfoLabel();
 #ifdef ECCODES_UI
@@ -1637,6 +1662,8 @@ void BufrExaminer::loadFilteredEmptyFile(QString f)
 
     Q_ASSERT(filteredPanel_);
     Q_ASSERT(filteredData_);
+    Q_ASSERT(data_);
+
     filteredPanel_->clear();
 
     //Set the new file name
@@ -1649,7 +1676,7 @@ void BufrExaminer::loadFilteredEmptyFile(QString f)
     actionSave_->setEnabled(false);
 
     //set control panel
-    filteredPanel_->setFilterInfo(true,sourceMessageNum_,0);
+    filteredPanel_->setFilterInfo(true,data_->totalMessageNum(),0);
 
 #ifdef ECCODES_UI
     filePanel_->setEnabled(true);
@@ -1674,13 +1701,36 @@ void BufrExaminer::slotUncompressedMode(bool)
     //mainPanel_->setNavigateUncompressed(actionUncompressedMode_->isChecked());
 }
 
+void BufrExaminer::newMessageLoaded(int msg)
+{
+    if(filterDialog_)
+    {
+        if(MvQBufrMainPanel* panel=static_cast<MvQBufrMainPanel*>(sender()))
+        {
+            filterDialog_->updateHelpers(panel->data(),
+                                         msg);
+        }
+    }
+}
+
 void BufrExaminer::slotFilterEdit()
 {
     if(!filterDialog_)
     {
        filterDialog_=new BufrFilterDialog(this);
        filterDialog_->setModal(false);
-       filterDialog_->setDecoder(static_cast<BufrMetaData*>(data_));
+
+       //Init helpers
+       filterDialog_->initHelpers(mainPanel_->data(),
+                                  mainPanel_->currentMessage());
+
+       if(filteredPanel_)
+       {
+           filterDialog_->updateHelpers(filteredPanel_->data(),0);
+           filterDialog_->updateHelpers(filteredPanel_->data(),
+                                        filteredPanel_->currentMessage());
+       }
+
        connect(filterDialog_,SIGNAL(edited(const BufrFilterDef&)),
                this,SLOT(slotRunFilter(const BufrFilterDef&)));
     }
@@ -1711,6 +1761,8 @@ void BufrExaminer::slotFilterRevert()
 
 void BufrExaminer::slotRunFilter(const BufrFilterDef& def)
 {  
+    Q_ASSERT(data_);
+
     //if the message filter is empty we revert to the
     //source file
     std::string err;
@@ -1722,7 +1774,7 @@ void BufrExaminer::slotRunFilter(const BufrFilterDef& def)
     BufrFilterEngine filter(sourceFileName_.toStdString(),BufrFilterEngine::EccodesMode,this);
 
     //Create progress bar
-    int num=sourceMessageNum_;
+    int num=data_->totalMessageNum();
     int minDuration=1000.;
     if(num > 1000)
     {
@@ -1739,9 +1791,13 @@ void BufrExaminer::slotRunFilter(const BufrFilterDef& def)
     //The object to store the extracted values
     MvKeyProfile* extractedProf=new MvKeyProfile("result");
 
+
     try
     {
-        filter.run(def,resFile.path(),extractedProf);
+        QTime timer;
+        timer.start();
+        filter.run(def,resFile.path(),extractedProf,num);
+        qDebug() << "elapsed: " << timer.elapsed();
 
         StatusMsgHandler::instance()->done();
 
@@ -1790,7 +1846,7 @@ void BufrExaminer::slotRunFilter(const BufrFilterDef& def)
     }
 }
 
-void BufrExaminer::notifyObsIteratorProgress(int n)
+void BufrExaminer::notifyBufrFilterProgress(int n)
 {
     Q_EMIT filterProgess(n);
 }
@@ -1865,13 +1921,19 @@ void BufrExaminer::slotStatusMessage(QString s)
 void BufrExaminer::updateFileInfoLabel()
 {
     //file info panel
-    fileInfoLabel_->setBufrTextLabel(sourceFileName_,sourceMessageNum_,false,data_->totalMessageNum(),false);
+    fileInfoLabel_->setBufrTextLabel(sourceFileName_,data_->totalMessageNum(),false,data_->totalMessageNum(),false);
 
     //window title
     //updateWinTitle(sourceFileName_ + ((filtered_)?" [filtered]":""));
     updateWinTitle(sourceFileName_);
 }
 
+void BufrExaminer::initFilterEditorHelpers()
+{
+    if(filterDialog_)
+        filterDialog_->initHelpers(mainPanel_->data(),mainPanel_->currentMessage());
+}
+
 void BufrExaminer::writeSettings()
 {
     QSettings settings("ECMWF",settingsName_);
