@@ -205,7 +205,6 @@ void MvQFolderViewBase::handleIconShortCut(QShortcut* sc,QModelIndexList indexLs
 	if(objLst.size() > 0)
 	{
 		QString selection=sc->property("cmd").toString();
-		qDebug() << "icon shortcut  selection" << selection;
 		if(!selection.isEmpty())
 		{  
 			handleIconCommand(objLst,selection);
@@ -413,6 +412,15 @@ void MvQFolderViewBase::handleDesktopCommand(QString name,QPoint pos,QWidget *wi
 			kind.createOne(folder,pos.x(),pos.y());
 		
 	}
+    else if(name == "createPython")
+    {
+        const IconClass &kind=IconClass::find("PYTHON");
+        if(ignoreItemPositionForCm())
+            kind.createOne(folder);
+        else
+            kind.createOne(folder,pos.x(),pos.y());
+
+    }
 	else if(name == "createIcon")
 	{  
 		MvQNewIconDialog dialog(widget);
