@@ -40,6 +40,11 @@ class BufrPicker : public MvService
 
    void serve( MvRequest&, MvRequest& );
 
+   // Remove this code when BufrFilter can handle Geographical
+   // Polar Vectors and XY Vectors
+#if 1
+   void serve_old( MvRequest&, MvRequest& );
+
  protected:
    void getParams();
    void getData();
@@ -74,6 +79,10 @@ class BufrPicker : public MvService
    double                   pickedValue2_;
    bool                     pickAll_;              //-- pick all values related to the innermost coordinate descriptor
    std::vector<std::string> geoVectorName_;        //-- [0]:empty, [1]:"POLAR_VECTOR", [2]:"XY_VECTOR"
+#endif
+
+protected:
+   void getInputParams(MvRequest&,MvRequest&);
 };
 
 
@@ -597,16 +606,11 @@ BufrPicker::convertDescriptorsToKeys(MvObs& obs)
       coordDescr_[i] = obs.keyC(coordDescr_[i]);
 }
 
-//_________________________________________________________close
+//_________________________________________________________serve_old
+// Remove this function when BufrFilter can handle Geographical
+// Polar Vectors and XY Vectors
 void
-BufrPicker::close()
-{
-   outfile_.close();
-}
-
-//_________________________________________________________serve
-void
-BufrPicker::serve( MvRequest& in, MvRequest& out )
+BufrPicker::serve_old( MvRequest& in, MvRequest& out )
 {
    time_t myStartTime;
    time( &myStartTime );
@@ -664,6 +668,137 @@ BufrPicker::serve( MvRequest& in, MvRequest& out )
    //out.print();
 }
 
+//_________________________________________________________close
+void
+BufrPicker::close()
+{
+   outfile_.close();
+}
+
+
+/****************************************************************
+ NEW CODE
+***************************************************************/
+
+//_____________________________________________________getInputParams
+void
+BufrPicker::getInputParams(MvRequest& in, MvRequest& out)
+{
+   // Get data
+   out("DATA") = (MvRequest)in.getSubrequest("DATA");
+
+   // Get output format
+   const char* outStr = in("OUTPUT");
+   if( outStr && (std::string(outStr) == geoVectorName_[1]) )
+      out("OUTPUT") = "POLAR_VECTOR";
+   else if( outStr && (std::string(outStr) == geoVectorName_[2]) )
+      out("OUTPUT") = "XY_VECTOR";
+   else
+      out("OUTPUT") = "GEOPOINTS";
+
+   //Get parameter values
+   MvObs obs;
+   int nparams = in.countValues("PARAMETER");
+   out("PARAMETER_COUNT") = nparams;
+   for ( int i = 0; i < nparams; i++ )
+   {
+      // Convert descriptor to key
+      string spar1 = (const char*)in("PARAMETER", i);
+      string spar2 = obs.keyC(spar1);
+
+      // Add parameter to the request
+      ostringstream sparam;
+      sparam << "PARAMETER_" << i+1 << ends;
+      out.setValue(sparam.str().c_str(), spar2.c_str());
+   }
+
+   // Get missing data info
+   const char* myMissingData = (const char*)in("MISSING_DATA");
+   if( myMissingData && strcmp(myMissingData, "INCLUDE") == 0 )
+   {
+      out("MISSING_DATA") = "INCLUDE";
+      out("MISSING_DATA_VALUE") = (double)in("MISSING_DATA_VALUE");
+   }
+
+   // Check coordinate descriptors and coordinate values
+   int ncoords = in.countValues("COORDINATE_DESCRIPTORS");
+   if( ncoords < 1 )
+      throw MvException("No coordinate descriptors given");
+
+   if( in.countValues("COORDINATE_VALUES") != ncoords )
+      throw MvException( "Different number of coordinate descriptors and values" );
+
+   // Get coordinate descriptors and coordinate values
+   bool extractAll = false;
+   out("COORDINATE_COUNT") = ncoords;
+   for ( int i = 0; i < ncoords; i++ )
+   {
+      // Convert descriptor to key
+      string spar1 = (const char*)in("COORDINATE_DESCRIPTORS", i);
+      string spar2 = obs.keyC(spar1);
+
+      // Add descriptor to the request
+      ostringstream sparam;
+      sparam << "COORDINATE_" << i+1 << ends;
+      out.setValue(sparam.str().c_str(), spar2.c_str());
+
+      // Check for value ALL
+      string coordValStr = (const char*)in( "COORDINATE_VALUES", i );
+      if( coordValStr == "ALL" )
+         extractAll = true;
+      else
+      {
+         ostringstream sparam1;
+         sparam1 << "COORDINATE_VALUE_" << i+1 << ends;
+         out(sparam1.str().c_str()) = (const char*)in("COORDINATE_VALUES", i);
+      }
+   }
+   out("EXTRACT_MODE") = extractAll ? "ALL" : "FIRST";
+
+   // Get tag informing how to proceed if an error occurs
+   const char* fail = in("FAIL_ON_ERROR");
+   if( fail && strcmp( fail, "NO" ) == 0 )
+      failOnError_ = false;
+}
+
+//_________________________________________________________serve
+void
+BufrPicker::serve( MvRequest& in, MvRequest& out )
+{
+in.print();
+#if 1
+   // Remove this code when BufrFilter can handle Geographical
+   // Polar Vectors and XY Vectors
+   const char* outStr = (const char*)in( "OUTPUT" );
+   if ( outStr && ((std::string(outStr) == geoVectorName_[1]) || ( std::string(outStr) == geoVectorName_[2])) )
+   {
+      serve_old(in,out);
+      return;
+   }
+#endif
+
+   try
+   {
+      // Get user input parameters
+      MvRequest bufrIn("BUFRFILTER");
+      MvRequest filterIn = bufrIn.ExpandRequest("BufrFilterDef","BufrFilterRules",EXPAND_DEFAULTS);
+      getInputParams(in,filterIn);
+
+      callService("BufrFilter",filterIn,out);
+   }
+   catch ( MvException& e )
+   {
+      marslog( LOG_WARN, "BufrPicker: %s", e.what() );
+      if( failOnError_ )
+         throw MvException( "BufrPicker failed" );
+   }
+
+   // Clean irternal memory
+   clear();
+
+out.print();
+}
+
 //_________________________________________________________main
 int
 main( int argc, char **argv )
