@@ -432,6 +432,7 @@ MvObsSet::next( bool unpack )
    return MvObs( NULL );
 }
 
+
 //____________________________________________________________________ add
 void
 MvObsSet::add( MvObs& anObs )
@@ -559,6 +560,58 @@ bool MvObsSet::writeCompressed(MvObs *obs)
    return err;
 }
 
+bool MvObsSet::writeCompressed(MvObs *obs,const std::vector<int>& subsetVec)
+{
+    assert(obs);
+
+    if(!obs->compressData())
+        return false;
+
+    if(subsetVec.empty())
+        return false;
+
+    // Clone the input handle
+    codes_handle *cloneH = codes_handle_clone(obs->getHandle());
+    if(cloneH == NULL)
+    {
+        std::cout << "ERROR -  MvObsSet::write(MvObs&) -> could not clone field" << std::endl;
+        return false;
+    }
+
+   size_t arrSize= subsetVec.size();
+   long *subsetArr=new long[arrSize];
+   for(size_t i=0; i < subsetVec.size(); i++)
+        subsetArr[i]=subsetVec[i];
+
+   codes_set_long(cloneH,"unpack",1);
+   codes_set_long_array(cloneH,"extractSubsetList",subsetArr,arrSize);
+   codes_set_long(cloneH,"doExtractSubsets",1);
+
+   // Get the coded message in a buffer
+   const void *buffer = NULL;
+   size_t size = 0;
+   if (codes_get_message(cloneH, &buffer, &size))
+   {
+      std::cout << "ERROR -  MvObsSet::write(MvObs&) -> could not create a buffer message" << std::endl;
+
+      // Release the clone's handle
+      codes_handle_delete(cloneH);
+      delete subsetArr;
+      return false;
+   }
+
+   // Write the buffer to a file
+   bool err = write(buffer,size);
+
+   // Release the clone's handle
+   codes_handle_delete(cloneH);
+
+   delete subsetArr;
+
+   return err;
+}
+
+
 //____________________________________________________________________ messageCount
 // Returns the number of BUFR messages found in the file.
 // WARNING: a packed BUFR message may contain several observations
@@ -761,7 +814,8 @@ MvObsSetIterator::MvObsSetIterator(MvObsSet& s, bool useHeaderOnly) :
     _SelectState(SF_notSet),
     ObsSet(&s),
     useHeaderOnly_(useHeaderOnly),
-    observer_(0)
+    observer_(0),
+    filterProgressStep_(20)
 {
 }
 
@@ -812,6 +866,19 @@ MvObs MvObsSetIterator::operator() ( ENextReturn returnType )
     return current;
 }
 
+void MvObsSetIterator::setFilterProgressStep(int filterProgressStep)
+{
+    filterProgressStep_=filterProgressStep;
+    assert(filterProgressStep_ >=0);
+    if(filterProgressStep_ == 0)
+       filterProgressStep_=1;
+}
+
+MvObs MvObsSetIterator::nextMessage()
+{
+    return ObsSet->next(!useHeaderOnly_);
+}
+
 void MvObsSetIterator::next()
 {
    current = ObsSet->next(!useHeaderOnly_);
@@ -1403,8 +1470,11 @@ MvObsSetIterator::selectOk( MvObs *anObs ) const
 
 //____________________________________________________________________
 bool
-MvObsSetIterator::AcceptedObs( MvObs& anObs ) const
+MvObsSetIterator::AcceptedObs(MvObs& anObs) const
 {
+    if(!anObs)
+        return false;
+
     if( _NoFiltersSet )
        return true;
 
