@@ -360,6 +360,13 @@ MapView::UpdateView ( MvRequest& viewReq )
       // command: viewRequest_ = viewReq;
       Owner().UpdateView(viewReq);
 
+      // Update Coastlines
+      // The above function Owner().UpdateView(viewReq) is very generic
+      // and is called by the other Views. But, this View has an embeded
+      // Coastlines icon, which is not updated in the MvDataBase by the
+      // above function. 
+      Owner().ProcessCoastlines ( coastRequest, PlotMod::Instance().CalledFromMacro() );
+
       // Redraw this page
       Owner().RedrawIfWindow();
       Owner().NotifyObservers();
